﻿using System.Data;
using System.Windows.Forms;

using SampleApplication.Data;
using SampleApplication.DataHelper;

namespace SampleApplication.BaseController
{
	public delegate void FillDelegate(DataTable table);
	public delegate DataTableKeys GetKeysDelegate(DataRow row);
	public delegate void SearchDelegate(DataTable table, SearchCondition condition);

    public class ListBaseController<DS, DT, DR> : SampleApplication.BaseController.BaseController
		where DS : DataSet
		where DT : DataTable
		where DR : DataRow
	{
		private DS _dataSet;
		public DS DataSet
		{
			get { return _dataSet; }
			set { _dataSet = value; }
		}

		private DT _dataTable;
		public DT DataTable
		{
			get { return _dataTable; }
			set { _dataTable = value; }
		}

		private BindingSource _bindingSource;
		public BindingSource BindingSource
		{
			get { return _bindingSource; }
			set { _bindingSource = value; }
		}

		private FillDelegate _fill;
		public FillDelegate Fill
		{
			get { return _fill; }
			set { _fill = value; }
		}

		private GetKeysDelegate _getKeys;
		public GetKeysDelegate GetKeys
		{
			get { return _getKeys; }
			set { _getKeys = value; }
		}

		private SearchDelegate _search;
		public SearchDelegate Search
		{
			get { return _search; }
			set { _search = value; }
		}

        public void InitializeController(DS dataSet, DT dataTable, BindingSource bindingSource, FillDelegate fill, GetKeysDelegate getKey, SearchDelegate search)
        {
            _dataSet = dataSet;
            _dataTable = dataTable;
            _bindingSource = bindingSource;
            _fill = fill;
            _getKeys = getKey;
            _search = search;
        }

        public ListBaseController<DataSet, DataTable, DataRow> GetBaseController()
        {
            ListBaseController<DataSet, DataTable, DataRow> baseController = new ListBaseController<DataSet, DataTable, DataRow>();
            baseController.DataSet = _dataSet;
            baseController.DataTable = _dataTable;
            baseController.BindingSource = _bindingSource;
            baseController.Fill = _fill;
            baseController.GetKeys = _getKeys;
            baseController.Search = _search;
            return baseController;
        }
        
        public virtual DR GetCurrentRow()
		{
			return (DR)DataRowHelper.GetCurrentRow(BindingSource);
		}

		public virtual DataTableKeys GetCurrentKeys()
		{
			DR row = GetCurrentRow();
			if (row == null) return null;
			return GetKeys(row);
		}
	}
}
